import os

import fluxterm.defaults as defaults
import fluxterm.utils as utils


def make_commands():
    """
    Read in commands from yaml
    """
    data = utils.read_yaml(os.path.join(defaults.assets_dir, "flux-commands.yaml"))[
        "commands"
    ]
    lines = ["# Flux Commands"]
    for command in data:
        lines.append(f"\n## {command['name']}")
        for group in command["groups"]:
            lines.append(f"\n### {group['name']}\n")
            for example in group["items"]:
                lines.append(example["title"] + "\n")
                lines.append(f"\n```bash\n{example['command']}\n````")
    markdown = "\n".join(lines)
    return markdown, data



__author__ = "Vanessa Sochat"
__copyright__ = "Copyright 2024, Vanessa Sochat, HPCIC Developer Tools"
__license__ = "MIT"

__version__ = "0.0.1"
AUTHOR = "Vanessa Sochat"
AUTHOR_EMAIL = "vsoch@users.noreply.github.com"
NAME = "fluxterm"
PACKAGE_URL = "https://github.com/converged-computing/flux-term"
KEYWORDS = "Flux framework, HPC, workload manager, jobs queue, textual"
DESCRIPTION = "Terminal application to interact with Flux jobs, cheat sheets, code, and good fortune"
LICENSE = "LICENSE"

################################################################################
# Global requirements

INSTALL_REQUIRES = (
    ("textual-dev", {"min_version": None}),
    ("pyaml", {"min_version": None}),
    ("rich-pixels", {"min_version": None}),
)

TESTS_REQUIRES = (("pytest", {"min_version": "4.6.2"}),)
INSTALL_REQUIRES_ALL = INSTALL_REQUIRES + TESTS_REQUIRES



from fluxterm.command import make_commands
from textual.app import ComposeResult
from textual.screen import Screen
from textual.widgets import Footer, Header, MarkdownViewer

markdown, data = make_commands()


class DocsScreen(Screen):
    """
    The docs screen renders the flux cheat sheet as markdown.
    """

    def compose(self) -> ComposeResult:
        yield Header()
        yield MarkdownViewer(markdown=markdown)
        yield Footer()

    @property
    def markdown_viewer(self) -> MarkdownViewer:
        return self.query_one(MarkdownViewer)

    async def on_mount(self) -> None:
        self.markdown_viewer.focus()

    def action_toggle_table_of_contents(self) -> None:
        self.markdown_viewer.show_table_of_contents = (
            not self.markdown_viewer.show_table_of_contents
        )

    async def action_back(self) -> None:
        await self.markdown_viewer.back()

    async def action_forward(self) -> None:
        await self.markdown_viewer.forward()



#!/usr/bin/env python
import argparse
import sys

import fluxterm
from fluxterm.app import Fluxterm


def get_parser():
    parser = argparse.ArgumentParser(
        description="FluxTerm",
        formatter_class=argparse.RawTextHelpFormatter,
    )

    parser.add_argument(
        "--version",
        dest="version",
        help="show software version.",
        default=False,
        action="store_true",
    )

    subparsers = parser.add_subparsers(
        help="actions",
        title="actions",
        description="actions",
        dest="command",
    )

    # print version and exit
    subparsers.add_parser("version", description="show software version")

    # List installed modules
    subparsers.add_parser(
        "start",
        formatter_class=argparse.RawTextHelpFormatter,
        description="start fluxterm",
    )
    return parser


def run_fluxterm():
    """
    run_fluxterm is the entrypoint!
    """
    parser = get_parser()

    def help(return_code=0):
        version = fluxterm.__version__

        print("\nFluxterm v%s" % version)
        parser.print_help()
        sys.exit(return_code)

    # If an error occurs while parsing the arguments, the interpreter will exit with value 2
    args, _ = parser.parse_known_args()

    # Show the version and exit
    if args.command == "version" or args.version:
        print(fluxterm.__version__)
        sys.exit(0)

    # We don't do anything different for start, but added as a TBA
    # for arguments, maybe.
    start()


def start():
    app = Fluxterm()
    app.run()


if __name__ == "__main__":
    run_fluxterm()



from textual.app import ComposeResult
from textual.screen import Screen
from textual.widgets import DataTable, Footer, Header, Static
import fluxterm.defaults as defaults

try:
    import flux
    import flux.job

    handle = flux.Flux()
except ImportError:
    flux = None
    fortunes = [
        "If you want more help, check out <a href='https://flux-framework.readthedocs.org'>the Flux documentation</a>"
    ]


class JobsScreen(Screen):
    """
    Show running flux jobs in a table
    """

    def compose(self) -> ComposeResult:
        yield Header()
        yield FluxJobs()
        yield Footer()


class FluxJobs(Static):
    """
    Generate a table of flux jobs
    """

    def compose(self) -> ComposeResult:
        yield DataTable()

    def on_mount(self) -> None:
        table = self.query_one(DataTable)
        listing = flux.job.job_list(handle)

        # The first row are columns
        ROWS = [
            (
                "id",
                "user",
                "urgency",
                "priority",
                "state",
                "name",
                "cwd",
                "ntasks",
                "ncores",
                "duration",
                "nnodes",
                "ranks",
                "expiration",
            )
        ]
        for job in listing.get_jobinfos():
            j = job.to_dict()
            state = defaults.JOB_STATES.get(j["state"].lower()) or j["state"]
            ROWS.append(
                (
                    j["id"],
                    j["userid"],
                    j["urgency"],
                    j["priority"],
                    state,
                    j["cwd"],
                    j["ntasks"],
                    j["ncores"],
                    j["duration"],
                    j["nnodes"],
                    j["ranks"],
                    j["expiration"],
                )
            )
        table.add_columns(*ROWS[0])
        table.add_rows(ROWS[1:])



import os

here = os.path.dirname(os.path.abspath(__file__))
assets_dir = os.path.join(here, "assets")

GLOBAL_BINDINGS = [
    ("m", "toggle_dark", "Dark mode"),
    ("b", "switch_mode('browser')", "Code Browser"),
    ("c", "switch_mode('docs')", "Commands"),
    ("h", "switch_mode('help')", "Help"),
    ("q", "quit", "Quit"),
]

# It's confusing to distinguish job states from status, so I'll just
# provide lookups for all terms I find.
JOB_STATES = {
    "depend": "👀️",
    "sched": "📅️",
    "run": "🏃️",
    "cleanup": "🧹️",
    "inactive": "😴️",
    # These are "virtual" states that I'm not sure show up here
    "pending": "🦩️",
    "running": "🏃️",
    "active": "🟢️",
    # status
    "completed": "✅️",
    "failed": "🔴️",
    "cancelled": "✖️",
    "timeout": "⌛️",
}



from rich.syntax import Syntax
from rich.traceback import Traceback

import fluxterm.defaults as defaults
from textual.app import ComposeResult
from textual.containers import Container, VerticalScroll
from textual.screen import Screen
from textual.widgets import DirectoryTree, Footer, Header, Static


class CodeBrowser(Static):
    """
    A code browser widget
    """

    BINDINGS = [
        ("f", "toggle_files", "Files"),
    ] + defaults.GLOBAL_BINDINGS

    def compose(self) -> ComposeResult:
        # Only allow browsing the PWD
        path = "./"
        with Container():
            yield DirectoryTree(path, id="tree-view")
            with VerticalScroll(id="code-view"):
                yield Static(id="code", expand=True)

    def on_directory_tree_file_selected(self, event: DirectoryTree.FileSelected):
        """
        Called when the user click a file in the directory tree.
        """
        event.stop()
        code_view = self.query_one("#code", Static)
        try:
            syntax = Syntax.from_path(
                str(event.path),
                line_numbers=True,
                word_wrap=False,
                indent_guides=True,
                theme="github-dark",
            )
        except Exception:
            code_view.update(Traceback(theme="github-dark", width=None))
            self.sub_title = "ERROR"
        else:
            code_view.update(syntax)
            self.query_one("#code-view").scroll_home(animate=False)
            self.sub_title = str(event.path)

    def on_mount(self) -> None:
        self.query_one(DirectoryTree).focus()


class BrowserScreen(Screen):
    """
    A code browser widget
    """

    def compose(self) -> ComposeResult:
        yield Header()
        yield CodeBrowser()
        yield Footer()



import os

import fluxterm.defaults as defaults
import fluxterm.screen as screens
from textual.app import App
from textual.reactive import var

try:
    import flux
except ImportError:
    flux = None


class Fluxterm(App):
    """
    Flux in a terminal is flux term.

    Nobody here ever claimed to be good at naming things.
    - Computer Scientists
    """

    CSS_PATH = os.path.join(defaults.assets_dir, "fluxterm.tcss")

    # This is bindings keys (e.g., "d" to functions)
    BINDINGS = defaults.GLOBAL_BINDINGS
    MODES = {
        "browser": screens.BrowserScreen,
        "docs": screens.DocsScreen,
        "help": screens.HelpScreen,
    }

    if flux is not None:
        BINDINGS.append(("j", "switch_mode('jobs')", "Flux Jobs"))
        MODES["jobs"] = screens.JobsScreen

    def on_mount(self) -> None:
        self.switch_mode("browser")

    show_tree = var(True)

    def action_toggle_files(self) -> None:
        """
        Called in response to key binding.
        """
        self.show_tree = not self.show_tree

    def watch_show_tree(self, show_tree: bool) -> None:
        """
        Called when show_tree is modified.
        """
        self.set_class(show_tree, "-show-tree")

    # Actions are tied to bindings. E.g., "toggle_dark" with d -> action_toggle_dark
    def action_toggle_dark(self) -> None:
        """
        Toggle dark mode.
        """
        self.dark = not self.dark



import os
import random

from rich_pixels import Pixels

import fluxterm.defaults as defaults
from textual.app import ComposeResult
from textual.containers import Center, ScrollableContainer
from textual.screen import Screen
from textual.widgets import Button, Footer, Header, Static

try:
    import flux
    import flux.job
    from flux.cli.fortune import fortunes

    handle = flux.Flux()
except ImportError:
    flux = None
    fortunes = [
        "If you want more help, check out <a href='https://flux-framework.readthedocs.org'>the Flux documentation</a>"
    ]


class FluxBird(Static):
    """
    Render the flux bird!
    """

    def compose(self) -> ComposeResult:
        pixels = Pixels.from_image_path(
            os.path.join(defaults.assets_dir, "fluxbird-small.png")
        )
        yield Static(pixels)


class Fortune(Static):
    def compose(self) -> ComposeResult:
        with Center():
            yield Static(random.choice(fortunes), classes="wisdom")


class HelpScreen(Screen):
    CSS = """
    Screen {
        align: center middle;
        align: center bottom;
    }
    .again-button {
        text-align: center;
        content-align: center middle;
        border: round;
    }
    .wisdom {
        border: wide white round;
        width: 40;
        height: 9;
        text-align: center;
        content-align: center middle;
    }
    """

    def compose(self) -> ComposeResult:
        yield Header()
        yield ScrollableContainer(Fortune(), FluxBird(), id="fortunes")
        with Center():
            yield Button("Again!", id="fortune", classes="again-button")
        yield Footer()

    def on_button_pressed(self, event: Button.Pressed) -> None:
        """
        Triggers when the button presses and updates the fortune.
        """
        button_id = event.button.id
        if button_id == "fortune":
            fortune_display = self.query_one("#fortunes")
            wisdom = self.query("Fortune")
            bird = self.query("FluxBird")
            if wisdom:
                wisdom.last().remove()
            if bird:
                bird.last().remove()
            new_fortune = Fortune()
            new_bird = FluxBird()
            fortune_display.mount(new_fortune)
            fortune_display.mount(new_bird)
